"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableauApiClient = void 0;
const tableau_superagent_1 = require("../superagent/tableau-superagent");
const inversify_1 = require("inversify");
let TableauApiClient = class TableauApiClient {
    constructor(baseUrl) {
        this.baseUrl = baseUrl;
        this.DEFAULT_USER_SITE_ROLE = 'Viewer';
        this.DEFAULT_USER_AUTH_SETTING = 'TableauIDWithMFA';
        this.USERS_PAGINATION_PAGE_SIZE = 500;
        this.GROUPS_PAGINATION_PAGE_SIZE = 25;
        this.superAgent = new tableau_superagent_1.TableauSuperAgent();
        this.siteId = '';
        this.authToken = '';
    }
    async signIn(jwt, contentUrl) {
        var _a, _b, _c;
        const payload = {
            credentials: {
                jwt: jwt,
                site: {
                    contentUrl: contentUrl
                }
            }
        };
        const response = await this.superAgent.signinTableau(this.generateSignInUrl(), payload);
        if (!((_b = (_a = response === null || response === void 0 ? void 0 : response.credentials) === null || _a === void 0 ? void 0 : _a.site) === null || _b === void 0 ? void 0 : _b.id) || !((_c = response === null || response === void 0 ? void 0 : response.credentials) === null || _c === void 0 ? void 0 : _c.token)) {
            throw new Error('Failed to sign in to Tableau');
        }
        this.siteId = response.credentials.site.id;
        this.authToken = response.credentials.token;
        return response;
    }
    async signOut() {
        await this.superAgent.postTableau(this.generateSignOutUrl(), this.authToken, {});
        this.siteId = '';
        this.authToken = '';
    }
    async getUsersOnSite() {
        const result = await this.getUsers(this.generateSitesUsersUrl(), 1);
        if (!result) {
            throw new Error("Failed to get list of users on site from Tableau");
        }
        return result;
    }
    async getUsersInGroup(groupId) {
        const result = await this.getUsers(this.generateSitesGroupsUsersUrl(groupId), 1);
        if (!result) {
            throw new Error("Failed to get list of users in Tableau Reporting group from Tableau");
        }
        return result;
    }
    async getGroupsForUser(tableauUserId) {
        const result = await this.getGroups(this.generateSitesUserGroupsUrl(tableauUserId), 1);
        if (!result) {
            throw new Error("Failed to get groups for user on site from Tableau");
        }
        return result;
    }
    async getGroupsForUsers(tableauUserIds) {
        const result = {};
        for (const tableauUserId of tableauUserIds) {
            const userGroups = await this.getGroupsForUser(tableauUserId);
            result[tableauUserId] = userGroups;
        }
        return result;
    }
    async addUserToSite(userEmail) {
        var _a;
        const payload = {
            user: {
                name: userEmail,
                siteRole: this.DEFAULT_USER_SITE_ROLE,
                authSetting: this.DEFAULT_USER_AUTH_SETTING
            }
        };
        const response = await this.superAgent.postTableau(this.generateSitesUsersUrl(), this.authToken, payload);
        if (!((_a = response === null || response === void 0 ? void 0 : response.user) === null || _a === void 0 ? void 0 : _a.id)) {
            throw new Error('Failed to add user to Tableau site');
        }
        return response;
    }
    async addUserToGroup(tableauUserId, groupId) {
        var _a;
        const payload = {
            user: {
                id: tableauUserId
            }
        };
        const response = await this.superAgent.postTableau(this.generateSitesGroupsUsersUrl(groupId), this.authToken, payload);
        if (!((_a = response === null || response === void 0 ? void 0 : response.user) === null || _a === void 0 ? void 0 : _a.id)) {
            throw new Error('Failed to add user to Tableau group');
        }
        return response;
    }
    async removeUserFromSite(tableauUserId) {
        await this.superAgent.deleteTableau(this.generateSitesUserUrl(tableauUserId), this.authToken);
    }
    async removeUserFromGroup(tableauUserId, groupId) {
        await this.superAgent.deleteTableau(this.generateSitesGroupsUserUrl(groupId, tableauUserId), this.authToken);
    }
    async getGroups(url, pageNumber) {
        let groups = [];
        const response = await this.superAgent.getTableauQueryUsers(url, this.authToken, pageNumber, this.GROUPS_PAGINATION_PAGE_SIZE);
        groups = groups.concat(response.groups.group);
        const availableItems = response.pagination.totalAvailable;
        const pageSize = response.pagination.pageSize;
        const availablePages = Math.ceil(availableItems / pageSize);
        if (availablePages > pageNumber) {
            const nestedResponse = await this.getGroups(url, pageNumber + 1);
            groups = groups.concat(nestedResponse);
        }
        return groups;
    }
    async getUsers(url, pageNumber) {
        let users = [];
        const response = await this.superAgent.getTableauQueryUsers(url, this.authToken, pageNumber, this.USERS_PAGINATION_PAGE_SIZE);
        users = users.concat(response.users.user);
        const availableItems = response.pagination.totalAvailable;
        const pageSize = response.pagination.pageSize;
        const availablePages = Math.ceil(availableItems / pageSize);
        if (availablePages > pageNumber) {
            const nestedResponse = await this.getUsers(url, pageNumber + 1);
            users = users.concat(nestedResponse);
        }
        return users;
    }
    generateSignInUrl() {
        return `${this.generateAuthUrl()}/signin`;
    }
    generateSignOutUrl() {
        return `${this.generateAuthUrl()}/signout`;
    }
    generateAuthUrl() {
        return `${this.baseUrl}/auth`;
    }
    generateSitesUsersUrl() {
        return `${this.generateSitesUrl()}/users`;
    }
    generateSitesUserUrl(userId) {
        return `${this.generateSitesUsersUrl()}/${userId}`;
    }
    generateSitesUserGroupsUrl(userId) {
        return `${this.generateSitesUsersUrl()}/${userId}/groups`;
    }
    generateSitesGroupsUsersUrl(groupId) {
        return `${this.generateGroupsUrl(groupId)}/users`;
    }
    generateSitesGroupsUserUrl(groupId, userId) {
        return `${this.generateSitesGroupsUsersUrl(groupId)}/${userId}`;
    }
    generateGroupsUrl(groupId) {
        return `${this.generateSitesUrl()}/groups/${groupId}`;
    }
    generateSitesUrl() {
        return `${this.baseUrl}/sites/${this.siteId}`;
    }
};
TableauApiClient = __decorate([
    (0, inversify_1.injectable)(),
    __metadata("design:paramtypes", [String])
], TableauApiClient);
exports.TableauApiClient = TableauApiClient;
//# sourceMappingURL=tableau-api-client.js.map